/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: enr_etl.sas                                                                         |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: November 26, 2015                                                                           |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       Requires that DEM_ETL module completes:                                                       |
|         BRIDGE_ENR_SPANS macro                                                                      |
|         MSCDM enrollment table                                                                      |
|         Patkey table                                                                                |
|   Program outputs:                                                                                  |
|      Enr_Spans Table, filtered by PatKey                                                            |
|      Enr_FullCoverage_Spans_Bridged                                                                 |
|      Enr_Spans_&P (full coverage partitioned spans)                                                 |
|      Prevalent_Patients                                                                             |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

/* 9/21/15 - MR: */
/*           - Tighten up valid enrollment criteria to include only Y/N coverages */
/*           - Improve efficiency of checking for years with coverage */
/* 9/29/15 - MR: */
/*           - Loosen enrollment criteria of DPtemp.Enr_Spans to filter for Y/N coverages without */
/*             requiring a minimum of at least one Y coverage */
/* 10/06/15 - MR: */
/*            - Cap Enr_End to the MIN(Enr_End,&DP_MaxDate.) and Enr_Start to MAX(Enr_Start,&DP_MinDate.) */
/* 10/19/15 - MR: */
/*            - Use patient partitioning to speed up creation of ENR_ETL by a factor of 5 */
/* 10/30/15 - MR: */
/*            - Added dataset option "(keep=PatID Partition)" when setting lkpfile parameter value to */
/*              prevent creating unintentional variables during hash lookup */
/* 11/02/15 - MR: */
/*            - Set N_Partitions equal to MAX of (# PatKey patients/200,000) or (# of years of DP data) */
/*            - Save patient dataset with partition ID in DPTemp.PatKeyPartition */
/*            - Save partitioned patient datasets in DPTemp (DPTemp.PartitionPats_1-DPTemp.PartitionPats_N) */
/*            - Save N_Partitions macro variable in SAS dataset DPTemp.N_Partitions */
/* 11/23/15 - MR: */
/*            - Modify to create DPTemp.Enr_Spans_&P partitions in this program instead of in dx3 incident episodes */
/*            - Modify to create bridge spans inside of partition loop */
/*            - Modify to create views that interleave all partitions for enrollment: DPTemp.Enr_Spans and DPTemp.Enr_Spans_&P */
/* 12/07/15 -MR */
/*            - Modify name of bridged enrollment spans from DPTemp.Enr_Spans_Bridged to DPTemp.Enr_Bridged_Spans */
/* 01/26/16 -MR */
/*            - JIRA QCA0-2: View DPTemp.Enr_FullCoverage_Spans_Bridged failed QA due to incorrect sortedby variables. */
/*              Decided to remove the view as it is not being used to produce any outputs. */

** TASK SUMMARY ;
** ------------- ;
** 1- Create enrollment spans, filtered by PatKey ;
**    - Split enrollment files into patient partitions to speed up join performance ;
**    - Join each partition PatKey and Enrollment file together, trimming span dates ;
**    - Bridge full coverage partitioned spans ;
** 2- Create view that interleaves partitioned enrollment spans ;
** 3- Create Prevalent_Patients (Years) Table ;

** 1- Create enrollment spans, filtered by PatKey ;

  ** Retrieve N_Partitions metadata;
  data _null_;
    set DPTemp.N_Partitions;
    call symputx(Name, Value,'L');
  run;

  ** Split Enrollment into non-overlapping patient partitions;
  %partition_splits ( InDsn=INDATA.&ENRTABLE ,
                      OutDsn=&ENRTABLE._ ,
                      numsets=&N_Partitions ,
                      lkpfile=DPTemp.PatKeyPartition (keep=PatID Partition) ,
                      PatID=PatID
                    );

  %macro _ENR_SPANS_LOOP;
     %local P;
     %do P = 1 %to &N_Partitions;
        %put;
        %put ** FOR PARTITION &P:;
        %put ** Join demographics and enrollment and trim enrollment span dates;
        %put ** Note: It is very important that enr start/stop dates are trimmed here to fall;
        %put **       within DP min/max dates as downstream modules depend upon it.;
        %put;

        proc sql _method ;
          create table DPTemp.Enr_Spans_&P (rename=(Trim_Start=Enr_Start Trim_End=Enr_End )) as
             select B.PatKey,
                    MAX(A.Enr_Start,&DP_MinDate.) as Trim_Start length=4 format=&DateFMT.,
                    MIN(A.Enr_End,&DP_MaxDate.) as Trim_End length=4 format=&DateFMT.,
                    A.DrugCov, A.MedCov
             from &ENRTABLE._&P as A
                      inner join
                  DPTemp.PartitionPats_&P (sortedby=PatID) as B
             on (A.PatID = B.PatID) and MedCov in ('Y' 'N') and DrugCov in ('Y' 'N') and
                 (. < Enr_Start <= Enr_End) and (Enr_Start <= &DP_MaxDate and Enr_End >= &DP_MinDate)
            order by PatKey, Trim_Start, Trim_End
          ;
        quit ;

        %put;
        %put ** FOR PARTITION &P:;
        %put ** Call BRIDGE_ENR_SPANS macro to create FullCoverage Bridged Spans  ;
        %put;
        %BRIDGE_ENR_SPANS( INFILE=DPTemp.Enr_Spans_&P,
                     OUTFILE=DPTemp.Enr_Bridged_Spans_&P,
                     PATID=PatKey,
                     MEDCOV=Y,
                     DRUGCOV=Y,
                     ENRSTART=Enr_Start,
                     ENREND=Enr_End,
                     ENROLGAP=45
                    );

        %end;

     %put;
     %put ** 2- Interleave all of the partitioned unbridged spans into a view;
     %put;
     data DPTemp.Enr_Spans (sortedby=PatKey Enr_Start Enr_End) /view=DPTemp.Enr_Spans ;
       set %do P = 1 %to &N_Partitions;
             DPTemp.Enr_Spans_&P
           %end; ;
        by PatKey Enr_Start Enr_End;
     run;

  %mend;
  %_ENR_SPANS_LOOP


** 3- Create Prevalent_Patients (Years) Table ;
   data Prevalent_Patients ;
     set DPtemp.Enr_Spans ;

     retain Start_Year &DP_StartYear
            End_Year &DP_EndYear ;

     Start_Span_Year = year(Enr_Start) ;
     End_Span_Year = year(Enr_End) ;

     Max_Start_Year = max(Start_Year,Start_Span_Year) ;
     Min_End_Year = min(End_Year,End_Span_Year) ;

     length Year $4. ;
     do This_Year = Max_Start_Year to Min_End_Year ;
        if (End_Span_Year >= This_Year) and (Start_Span_Year <= This_year ) then do ;
          Year = put(This_Year,4.) ;
           output ;
        end ;
     end ;

     drop Start_Year End_Year Start_Span_Year End_Span_Year Max_Start_Year Min_End_Year This_Year Enr_Start Enr_End ;

   run ;

   proc sql _method ;
     create table DPtemp.Prevalent_Patients as
       select PatKey, year, max(MedCov) as MedCov length=1, max(DrugCov) as DrugCov length=1
         from Prevalent_Patients
         group by PatKey, year
         order by PatKey, year
       ;
   quit ;


** End enr_etl.sas ;
