/*---------------------------------------------------------------------------------------------------*\
|   Program name: BRIDGE_ENR_SPANS.SAS                                                                |
|                                                                                                     |
|   Date: 11/15/2015                                                                                  |
|   Version: 1.1                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   While allowing a maximum gap in enrollment, this program bridges member raw enrollment episodes   |
|   into continuous periods.                                                                          |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with enrollment data                                                |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT format) containing the bridged enrollment episodes       |
|                                                                                                     |
|   Macro variable definitions:                                                                       |
|   INFILE    = SAS dataset containing the raw enrollment data                                        |
|   OUTFILE   = SAS output dataset containing the conciliated enrollment periods                      |
|   PATID     = Name of patient-ID variable in INFILE                                                 |
|   MEDCOV    = Medical coverage indicator                                                            |
|   DRUGCOV   = Drug coverage indicator                                                               |
|   ENRSTART  = Name of enrollment episode start date variable in INFILE                              |
|   ENREND    = Name of enrollment episode end date variable in INFILE                                |
|   ENROLGAP  = Maximum allowable enrollment episode gap                                              |
|                                                                                                     |
|   Programming Notes:                                                                                |
|            -Consolidated enrollment episode start and end dates are named EStart EEnd.              |
|                   User can use rename after if other names are preferred.                           |
|            -The input file must have the MEDCOV and DRUGCOV variables.                              |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

/*  9/16/15 - MR: */
/*            -  Create modified copy of MS_EPIDODERECS.SAS MS-TOOLKIT MACRO: */
/*               -  Renamed macro to BRIDGE_ENR_SPANS.SAS */
/*               -  Parameterized the patient-id */
/*               -  Shortened length of date variables created to 4 bytes */
/*  9/21/15 - MR: */
/*            - Rework data step to avoid generating missing value messages */
/*            - Fix bug to test if input data are stored */
/* 10/06/15 - MR: */
/*            - Ensure spans are truly unique: */
/*              - Drop extraneous vars from output file, including MedCov and DrugCov */
/*              - Push out &ENREND if it overlaps a prior span */

%macro BRIDGE_ENR_SPANS(INFILE=,OUTFILE=,PATID=,MEDCOV=,DRUGCOV=,ENRSTART=,ENREND=,ENROLGAP=);

  %let medcovcond=1;
  %let drugcovcond=1;

  %if %eval((%upcase(&MEDCOV.) eq N or %upcase(&MEDCOV.) eq NO) and
        (%upcase(&DRUGCOV.) eq N or %upcase(&DRUGCOV.) eq NO))=0 %then %do;
    %if (%upcase(&MEDCOV.) eq Y or %upcase(&MEDCOV.) eq YES) %then %let medcovcond = upcase(MedCov)='Y';
    %if (%upcase(&MEDCOV.) eq N or %upcase(&MEDCOV.) eq NO) %then %let medcovcond = upcase(MedCov)='N';

    %if (%upcase(&DRUGCOV.) eq Y or %upcase(&DRUGCOV.) eq YES) %then %let drugcovcond = upcase(DrugCov)='Y';
    %if (%upcase(&DRUGCOV.) eq N or %upcase(&DRUGCOV.) eq NO) %then %let drugcovcond = upcase(DrugCov)='N';
  %end;

  %if %length(&PatID.) = 0 %then %do ;
     %let PatID = PatID ;
  %end ;

  %put ** Local macro parameters;
  %put _LOCAL_;

  *Determine if already sorted by PatId Enr_Start and Enr_End (extra sort vars beyond that are fine);
    %local SORTEDBY;
    proc contents noprint data=&INFILE. out=_contents_ (keep=name sortedby) ;
    run ;
    proc sql noprint ;
      select name into :SORTEDBY separated by ' '
        from _contents_
        where sortedby between 1 and 3
        order by sortedby
      ;
    quit ;
    %put ;
    %put &=SORTEDBY ;
    %put ;

  /* Macro variables WhichFile and WhichWhere are used by data step */
  /* below that creates enrollment episodes.  The values of the */
  /* "Which" macro variables are determined by whether the input */
  /* data is already sorted the way we wish */
  %local WhichFile WhichWhere;
  %if "%upcase(&SORTEDBY.)" = "%upcase(&PatID. &ENRSTART. &ENREND.)"  %then %do;
      %* Input data is already sorted in the proper order;
      %let WhichFile = &INFILE.;
      %let WhichWhere = where &medcovcond. and &drugcovcond.;
  %end ;
  %else %do;
      %let WhichFile = _denomint_;
      %let WhichWhere = ;
      %*Sort input data prior to creating episodes in next step*;
      proc sql _method;
        create table &WhichFile as
        Select *
        from &INFILE.
        where &medcovcond. and &drugcovcond.
        order by &PatID., &ENRSTART., &ENREND.
       ;
      quit;
  %end;


  *Create enrollment episodes;
  data _denomint_;
  set  &WhichFile;
  &WhichWhere ;
  by &PatID. &ENRSTART. &ENREND.;
  length Enr_episode 3. ;
  format last_&ENREND. date9.
  ; retain Enr_episode last_&ENREND. ;
  if first.&PATID. then do ;
     Enr_episode = 1;
     last_&ENREND. = .;
  end ;
  else do ;
     if (&ENRSTART.-last_&ENREND.-1) > &ENROLGAP. then Enr_episode=Enr_episode+1;
  end ;
  &ENREND. = MAX(&ENREND.,last_&ENREND.);
  last_&ENREND. = &ENREND. ;
  drop last_&ENREND. ;
  run;

  Proc SQL _method;
  Create Table &OUTFILE. as
  Select &PatID., Enr_episode,
         min(&ENRSTART.) as EStart format=mmddyy10. length=4,
         max(&ENREND.) as EEnd format=mmddyy10. length=4
  From _denomint_ as Enrol
  group by &PatID., Enr_episode;
  quit;

  proc datasets library = work Nodetails nolist nowarn;
  delete _contents_ _denomint_ ;
  quit;

%mend BRIDGE_ENR_SPANS;

* End of bridge_enr_spans.sas;