/*---------------------------------------------------------------------------------------------------*\
|    MACRO NAME: inc_next.sas                                                                         |
|     Performs administrative tasks for modules that run sequentially                                 |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: November 26, 2015                                                                           |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Macro inputs:                                                                                     |
|       Macro parameters:                                                                             |
|         CODE:   Name of input module to include next (without .SAS extension)                       |
|         SOURCE: Turn on or off printing source statements to log {values are: SOURCE|NOSOURCE}      |
|         LOG:    Optional LOG created for &CODE                                                      |
|         LPATH:  Log path for &CODE                                                                  |
|         CLEAN_WORK_B:    Kill work library before module is included (Y/N)                          |
|         CLEAN_WORK_A:    Kill work library after module is finished (Y/N)                           |
|         DEL_DPT_B:       Delete DPTemp files before module is included (Y/N)                        |
|         DEL_DPT_A:       Delete DPTemp files after module finishes (Y/N)                            |
|         DeleteDPT:       List of DPTemp files to delete. Specifying KillAll deletes library.        |
|         RESET_LOG_TO:    Option log to reset log to after module is finished                        |
|   Program outputs:                                                                                  |
|        Results of executed module, RC of executed module is used to control program flow, RunTime   |
|         of executed module printed to log and MSOC.ModuleDuration dataset, optional log file        |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

/* 9/21/15 - MR: */
/*           - Modify default value of CODE parameter to point to &MODULE_CODE so that it works */
/*             within new folder structure */
/*           - Add new program header */
/*           - Modify ERROR messages to log to only print once if &CODE specific log is not created */

/* The macro "inc_next" */
%MACRO INC_NEXT( code= ,
              cpath=%str(&module_code) ,
              source=source ,
              log=&ModuleLogs ,
              lpath=%str(&msoc) ,
              clean_work_b=&CleanWork ,
              clean_work_a=N ,
              del_DPT_b=N ,
              del_DPT_a=&CleanDPTemp ,
              DeleteDPT= ,
              reset_log_to=&_master_program_.
             )  ;

  /* Convert program name to lower case for UNIX compatibility */
  %let code = %lowcase(&code) ;

  /* Only include next module if no fatal errors previously encountered */
  %if %sysevalf(&syscc le 4) %then %do ;

     %put *** Starting Module: %upcase(&code);
     /* Start timer */
     %local module_start ;
     %TimeStamp(parm=module_start);

     /* If asked, create new log file for module */
     %if %upcase(&log) eq Y %then %do ;
        %put *** Create new log for module %upcase(&code);
        proc printto log="&lpath.&code._&DPID&SITEID..log" new;
        run;
     %end ;

     /* If asked, clear work library in preparation for module */
     %if %upcase(&clean_work_b) eq Y %then %do ;
       %put *** Clear work library in preparation for module %upcase(&code);
       proc datasets memtype=data lib=work nolist nodetails kill ;
       quit ;
     %end ;

     /* If asked, delete DPTemp files in preparation for module */
     %if %upcase(&del_DPT_b) eq Y and %length(&DeleteDPT) >0 %then %do;
        %if %upcase(&DeleteDPT) ne KILLALL %then %do;
           %put *** Delete DPTemp files selected in preparation for module %upcase(&code);
           proc datasets lib=DPTemp nolist nowarn nodetails;
              delete  &DeleteDPT. ;
           quit;
        %end;
        %else %do;
           %put *** Delete all remaining DPTemp files in preparation for module %upcase(&code);
           proc datasets lib=DPTemp kill nolist nowarn nodetails;
           quit;
        %end;
     %end;


     /* Include next module */
     %local rc ;
     %let syscc=0;
     %inc "&cpath.&code..sas" /&source ;
     %let rc = &syscc ;

     /* If module failed, execute termination code */
     %if %sysevalf(&rc gt 4) %then %do ;

        %if %length(&reset_log_to) > 0 %then %do ;
           /* Reset log to master module seen by MSOC */
           proc printto log="&lpath.&reset_log_to._&DPID&SITEID..log";
           run ;

           /* Put messages to the log for MSOC */
           %put ;
           %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
           %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
           %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
           %put ;
           %put RC for &cpath.&code..sas: &rc ;
           %put Execution terminating due to problems encountered. ;
           %if %upcase(&log) eq Y %then %do ;
              %put Please check "&lpath.&code..log" for details. ;
           %end ;
           %put ;
           %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
           %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
           %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
           %put ;
        %end ;

        /* Reset log to master program seen by site programmer */
        proc printto log=log;
        run;

        /* Repeat messages above ... then issue abort cancel */
        %put ;
        %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
        %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
        %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
        %put ;
        %put RC for &cpath.&code..sas: &rc ;
        %put Execution terminating due to problems encountered. ;
        %if %upcase(&log) eq Y %then %do ;
            %put Please check "&lpath.&code..log" for details. ;
        %end ;
        %put ;
        %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
        %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
        %put !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ;
        %put ;
        %abort cancel 99 ;
     %end ;

     /* Else if execution was successful, execute post-module code */


     /* If asked, clear work library in preparation for next module */
     %if %upcase(&clean_work_a) eq Y %then %do ;
       %put *** Clean work library in preparation for next module;
       proc datasets memtype=data lib=work nolist nodetails kill ;
       quit ;
     %end ;

     /* If asked, delete DPTemp files */
     %if %upcase(&del_DPT_a) eq Y and %length(&DeleteDPT) >0 %then %do;
        %if %upcase(&DeleteDPT) ne KILLALL %then %do;
           %put *** Delete DPTemp files selected in preparation for next module;
           proc datasets lib=DPTemp nolist nowarn nodetails;
              delete  &DeleteDPT. ;
           quit;
        %end;
        %else %do;
           %put *** Delete all remaining DPTemp files in preparation for next module;
           proc datasets lib=DPTemp kill nolist nowarn nodetails;
           quit;
        %end;
     %end;

     /* Stop timer */
     %local module_end ;
     %TimeStamp(parm=module_end);

     /* Output to SAS dataset module runtime stats */
      %UpDateSigFile(
      ProcessName=%upcase(&code) ,      /* The name of the process  (50 chars)                 */
      ProcessRC=&RC ,                   /* Return code associated with process name            */
      StartTime=&module_start,          /* The start time for a process as SAS date/time value */
      EndTime=&module_end,              /* The end time for a process as SAS date/time value   */
      OutSigName=MSOC.ModuleDuration    /* Name of output signature dataset                    */
      );

     %if %length(&reset_log_to) > 0 %then %do ;
       ** Reset log to master module log;
       proc printto log="&lpath.&reset_log_to._&DPID&SITEID..log";
       run ;
     %end ;

     /* Output to log the RC and runtime stats for the module that just finished */
     %put <<<<<<<<<<<<<<< Module %upcase(&code) Status Report >>>>>>>>>>>>>>>> ;
     %put ;
     %put RC for &cpath.&code..sas: &rc ;
     %put Module %upcase(&code) finished without fatal issues detected.;
     %put ;
     %TimeReport(module_start, module_end) ;
     %put ;

  %end ;
%MEND INC_NEXT ;

* End of inc_next.sas;