/*---------------------------------------------------------------------------------------------------*\
|    MACRO NAME: incident_rx_episodes.sas                                                             |
|      Processes dispensings to create incident dispensing episodes for each                          |
|      lookback period (90, 180, 270)                                                                 |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: November 26, 2015                                                                           |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

/* MOD: 10/15/15 -MR: */
/*              1) BUG FIX: Keep only the first incident episode per year */
/*              2) Create an incident episode constant for use by INCIDENT_RX_SUMMM macro */
/* MOD: 10/30/15 -MR: */
/*              1) BUG FIX: Fix "Off-By-1" that added an additional day to LookBack calc */
/*              2) BUG FIX: Add ORDER BY statement to ensure sort order of "rescue" claims */
/* MOD: 11/09/15 -MR: */
/*              1) Add comments on trimming claims to fall within DP min/max dates */
/*              2) Remove PercentDays parameter from STOCKPILE_RX macro call */
/*              3) Re-calculate AgeGroup for "rescued" claims that jumped enr spans */
/* MOD: 11/15/15 -MR: */
/*              1) Modify to split the dispensings into patient partitions and process */
/*                 the rest of the program steps inside a partition loop*/
/* MOD: 11/23/15 -MR: */
/*              1) Shorten dataset names to allow a partition suffix of <= 999 */
/* 12/07/15 -MR */
/*            - Modify name of bridged enrollment spans from DPTemp.Enr_Spans_Bridged.. to DPTemp.Enr_Bridged_Spans.. */

** TASK SUMMARY ;
** -------------;
** 0- SPLIT DISPENSING DATA INTO PATIENT PARTITIONS;
** 1- LINK DISPENSING CLAIMS FOR EACH PARTITION;
**     - link with enrollment spans;
** 2- CLEAN DISPENSING CLAIMS FOR EACH PARTITION;
**     - exclude (if not previously done) invalid dispensings;
**     - aggregate same-day claims keeping max RxSup, assuming same-day fills are consumed concurrently;
** 3- STOCKPILE DISPENSING CLAIMS FOR EACH PARTITION;
**     - use utility macro STOCKPILE_RX with unconditional algorithm (i.e. always forward fill);
** 4- CLEAN STOCKPILED DISPENSING CLAIMS FOR EACH PARTITION;
**     - exclude fills that get pushed past end of enrollment;
**     - truncate ExpireDt and RxSup if pushed past enrollment;
** 5- CREATE RX EPISODES FOR EACH PARTITION;
**     - Use utility macro CREATE_RX_EPISODE_CLAIMS to assign episode number to dispensings;
**     - Dispensings with <=15 day gaps are assigned to same episode;
**     - Rollup dispensings with into unique episodes;
** 6- CREATE INCIDENT EPISODES FOR 90, 180, and 270 DAY LOOKBACK PERIODS FOR EACH PARTITION;
**     - Identify candidate epsiodes that meet minimum lookback criteria;
**     - Select 1st incident episode for each year, outputting an episode record for every;
**       lookback criteria the incident episode meets;

%macro Incident_Rx_Episodes
      ( InFile=,             /*Input file prefix (Note: include libname if necessary)*/
        OutFile=,            /*Output file name (Note: include libname) */
        DrugCategory= ,      /*Drug Category Name field: {GenericName | DrugClass}*/
        DrugCategoryKey= ,   /*Drug Category Key field: {GenericNameKey | DrugClassKey}*/
        DEBUG=N              /*Y/N flag for creating extra output (Default is N)*/
       ) ;

%put;
%put *** TASK 0- SPLIT DISPENSING DATA INTO PATIENT PARTITIONS;
%put;

  ** Retrieve N_Partitions;
  data _null_;
    set DPTemp.N_Partitions;
    call symputx(Name, Value,'L');
  run;

  ** Split dispensing claims time-period subsets into patient partition subsets;
  %partition_splits ( InDsn=&InFile.: ,
                      OutDsn=&DrugCategory._ ,
                      numsets=&N_Partitions ,
                      lkpfile=DPTemp.PatKeyPartition (keep=PatKey Partition) ,
                      PatID=PatKey
                      );

%put ;
%put ************** BEGIN PARTITION LOOP PROCESSING ******************;
%put ;
  %local P;
  %DO P = 1 %to &N_Partitions;  /* BEGIN PARTITION LOOP */
     %put;
     %put *** FOR PARTITION &P , ;
     %put *** TASK 1- LINK DISPENSING CLAIMS TO ENROLLMENT;
     %put;

     *** Dispensing RxDate must fall within a bridged enrollment span;
     PROC SQL _method;
       CREATE TABLE RX_&DrugCategory._&P as
         SELECT rx.PatKey , rx.&DrugCategoryKey. , rx.RxDate , rx.RxSup ,
                rx.Sex, rx.AgeEventGroupKey , enr.EStart , enr.EEnd
           FROM &DrugCategory._&P as rx
                  INNER JOIN
                DPTemp.Enr_Bridged_Spans_&P as enr
             ON rx.PatKey = enr.PatKey and (enr.EStart <= rx.RxDate <= enr.EEnd)
         ORDER BY rx.PatKey , rx.&DrugCategoryKey. , rx.RxDate
          ;
     QUIT ;

     %if %upcase(&DEBUG.) ne Y %then %do;
        *** Delete datasets no longer needed;
        proc datasets library=work nolist nowarn nodetails;
           delete &DrugCategory._&P ;
        quit;
     %end;

     %put;
     %put *** FOR PARTITION &P , ;
     %put *** TASK 2- CLEAN DISPENSING CLAIMS;
     %put;

     *** Aggregate same-day claims, keeping max RxSup;
     %** DEV Note: combining aggregating with joining steps together in SQL performed worse;
     DATA RX_&DrugCategory._Clean_&P (sortedby=PatKey &DrugCategoryKey. RxDate);
        SET RX_&DrugCategory._&P (rename=(RxSup=_RxSup));
        BY PatKey &DrugCategoryKey. RxDate;
        length RxSup 4. NumClaims 3.; retain NumClaims RxSup .;
        if first.RxDate then do;
          RxSup=_RxSup;
          NumClaims=1;
        end;
        else do;
          RxSup=MAX(RxSup,_RxSup);
          NumClaims+1;
        end ;

        if last.RxDate;
        drop _RxSup;
     RUN ;

     %if %upcase(&DEBUG.) ne Y %then %do;
        *** Delete datasets no longer needed;
        proc datasets library=work nolist nowarn nodetails;
           delete RX_&DrugCategory._&P;
        quit;
     %end;

     %put;
     %put *** FOR PARTITION &P , ;
     %put *** 3- STOCKPILE DISPENSING CLAIMS;
     %put;

     *** Create stockpiled dispensings;
     %STOCKPILE_RX
      ( INFILE=RX_&DrugCategory._Clean_&P, /*Input file with drug claims data*/
        PATID=PatKey,                /*Name of variable in INFILE used to identify the Patient-ID*/
        CLMDATE=RxDate,              /*Name of variable in INFILE used to identify the claim date*/
        CLMSUP=RxSup,                /*Name of variable in INFILE to identify the claim days of supply*/
        CLMAMT=,                     /*Name of variable in INFILE to identify the claim amount dispensed (OPTIONAL)*/
        GROUPING=&DrugCategoryKey.,  /*Variables in INFILE used to group claims*/
        WHERE=,                      /*Condition for applicable rows in INFILE (OPTIONAL*/
        OUTFILE=RX_&DrugCategory._Stockpiled_&P.,  /*Output file*/
        DEBUG=&DEBUG.                /*Debug turns on prints and keeps extra variables (OPTIONAL)*/
      );

     %if %upcase(&DEBUG.) ne Y %then %do;
        *** Delete datasets no longer needed;
        proc datasets library=work nolist nowarn nodetails;
           delete RX_&DrugCategory._Clean_&P ;
        quit;
     %end;

     %put;
     %put *** FOR PARTITION &P , ;
     %put *** 4- CLEAN STOCKPILED DISPENSING CLAIMS;
     %put;

     *** Split stockpiled claims into a save pile and exclude pile, trimming ExpireDt and RxSup if necessary ;
     *** QC Note: Enr start/stop dates were trimmed by DP min/max dates in the ENR_ETL module;
     DATA RX_&DrugCategory._Stockpiled_&P (sortedby=PatKey &DrugCategoryKey. RxDate)
          RX_&DrugCategory._Exclude_&P (sortedby=PatKey &DrugCategoryKey. RxDate rename=(EStart=OrigEStart EEnd=OrigEEnd));
       SET RX_&DrugCategory._Stockpiled_&P ;
       ** Split out stockpiled claims whose &CLMDATE falls after EEnd to an exclude file;
       if (RxDate > EEnd) then do ;
          output RX_&DrugCategory._Exclude_&P ; delete ;
       end ;
       ** Truncate &CLMSUP and ExpireDt to fall inside of EEnd;
       if (ExpireDt > EEnd) then do ;
          ExpireDt = MIN(ExpireDt, EEnd) ;
          RxSup = (ExpireDt - RxDate)+1 ;
       end ;
       ** Output surviving dispensings;
       output RX_&DrugCategory._Stockpiled_&P ;
     RUN ;

     %if %DSN_OBS(RX_&DrugCategory._Exclude_&P) > 0 %then %do;
        *** Rescue (if possible) dispensings that were pushed past their original enrollment span;
        *** and "landed" onto a future enrollment span and re-calculate AgeGroup for rescued claims;

        PROC SQL _method;
           CREATE TABLE RX_&DrugCategory._Rescue_&P as
             SELECT rx.* ,
                    enr.EStart , enr.EEnd,
                    put(max(0,AgeCalc(dem.Birth_Date, rx.RxDate)),AgeGroup_to_ID.) as AgeEventGroupKey
             FROM DPTemp.Enr_Bridged_Spans_&P as enr ,
                  DPTemp.PartitionPats_&P as dem ,
                  RX_&DrugCategory._Exclude_&P (drop=OrigEStart OrigEEnd AgeEventGroupKey) as rx
               WHERE rx.PatKey = enr.PatKey and (enr.EStart <= rx.RxDate <= enr.EEnd) and enr.PatKey = dem.PatKey
               ORDER BY rx.PatKey , rx.&DrugCategoryKey. , rx.RxDate
           ;
        QUIT ;
        %if %DSN_OBS(RX_&DrugCategory._Rescue_&P) > 0 %then %do;
           *** Interleave "rescued" claims back with rest of the stockpiled claims;
           *** QC Note: Enr start/stop dates were trimmed by DP min/max dates in the ENR_ETL module;
           DATA RX_&DrugCategory._Stockpiled_&P (sortedby=PatKey &DrugCategoryKey. RxDate);
             SET RX_&DrugCategory._Stockpiled_&P
                 RX_&DrugCategory._Rescue_&P (in=in_rescue);
             BY PatKey &DrugCategoryKey. RxDate;
             if in_rescue then do;
               ** Truncate &CLMSUP and ExpireDt to fall inside of EEnd;
               if (ExpireDt > EEnd) then do ;
                 ExpireDt = MIN(ExpireDt, EEnd) ;
                 RxSup = (ExpireDt - RxDate)+1 ;
               end ;
             end;
           RUN;
        %end;
        %if %upcase(&DEBUG.) ne Y %then %do;
           *** Delete datasets no longer needed;
           proc datasets library=work nolist nowarn nodetails;
             delete &DrugCategory._Exclude_&P
                    RX_&DrugCategory._Rescue_&P;
          quit;
       %end;
     %end;

     %put;
     %put *** FOR PARTITION &P , ;
     %put *** 5- CREATE RX EPISODES;
     %put;

     *** Assign episode number to dispensing claims ;
     %CREATE_RX_EPISODE_CLAIMS
       ( INFILE=RX_&DrugCategory._Stockpiled_&P,
         GROUPING=&DrugCategoryKey.,
         GAP=15,
         PATID=PatKey,
         CLMDATE=RxDate,
         EXPIREDT=ExpireDt,
         ENRSTART=EStart,
         OUTFILE=RX_&DrugCategory._EClaims_&P,
         DEBUG=&DEBUG.
       );

     *** Roll up episodes;
     proc SQL _method;
       CREATE TABLE RX_&DrugCategory._episodes_&P as
         SELECT PatKey, &DrugCategoryKey., episode,
                min(Sex) as Sex,
                min(AgeEventGroupKey) as AgeEventGroupKey,
                sum(NumClaims) as EpiNumClaims length=3,
                sum(RxSup) as EpiRxSup length=4,
                min(RxDate) as EpiStart length=4 format=date9.,
                max(ExpireDt) as EpiEnd length=4 format=date9.,
                min(EStart) as EStart length=4 format=date9.
                %if %upcase(&DEBUG.) eq Y %then %do;
                   , max(EEnd) as EEnd length=4 format=date9.
                %end;
            FROM RX_&DrugCategory._EClaims_&P
            GROUP BY PatKey, &DrugCategoryKey., episode
          ;
     quit;
     %if %upcase(&DEBUG.) ne Y %then %do;
        *** Delete datasets no longer needed;
        proc datasets library=work nolist nowarn nodetails;
          delete RX_&DrugCategory._EClaims_&P;
        quit;
     %end;

     %put;
     %put *** FOR PARTITION &P , ;
     %put *** 6- CREATE INCIDENT EPISODES FOR 90, 180, and 270 DAY LOOKBACK PERIODS;
     %put;


     *** Identify candidate episodes that meet minimum Lookback criteria;
     *** QC Note: Enr start/stop dates were trimmed by DP min/max dates in the ENR_ETL module;
     DATA candidates_RX_&DrugCategory._&P;
       SET RX_&DrugCategory._episodes_&P;
       BY PatKey &DrugCategoryKey. Episode;

       length year $4. qtr $1.;
       year=put(year(EpiStart),4.);
       qtr=put(qtr(EpiStart),1.);

       length EpiDuration 4.;
       EpiDuration = (EpiEnd-EpiStart)+1;

       ** Calc prior episode end date;
       length PriorEpiEnd 4.; format PriorEpiEnd date9.;
       PriorEpiEnd=lag(EpiEnd);
       if first.&DrugCategoryKey. then PriorEpiEnd=.;

       ** Test if episode meets minimum LookBack criteria;
       length MinLookBackDt 4.; format MinLookBackDt date9.;
       MinLookBackDt = (EpiStart-90);
       if (EStart <= MinLookBackDt) and (PriorEpiEnd < MinLookBackDt) ;

       %if %upcase(&DEBUG.) eq N %then %do;
          drop MinLookBackDt;
       %end;
     RUN;

     *** Within each year, select the 1st incident DrugCategory episode for each patient;
     *** Determine if incidence criteria is met for 90, 180, and 270 days lookback periods.;
     *** For each lookback period the episode qualifies, output an episode "lookback period" record.;
     *** Likewise, for every non-qualifying episode, output to an exclude dataset if DEBUG is turned on;
     *** QC Note: Enr start/stop dates were trimmed by DP min/max dates in the ENR_ETL module;
     DATA &OUTFILE._&P
          %if %upcase(&DEBUG.) eq Y %then %do;
             RX_&DrugCategory._epi_excl_&P
          %end;
         ;
       set candidates_RX_&DrugCategory._&P;
       by PatKey &DrugCategoryKey. Year Episode;

       ** Select the 1st incident qualifying episode within a year;
       if first.Year;

       ** The macro INCIDENT_RX_SUMM uses an incident episode constant to produce member counts more efficiently;
       length Incident_Episode 3.; retain Incident_Episode 1;

       ** Output incident episode for each lookback period it qualifies for;
       length LookBack 3. LookBackDt 4.; format LookBackDt date9.;
       do LookBack = 90 to 270 by 90;
         LookBackDt = (EpiStart-LookBack);
         if (EStart <= LookBackDt) and (PriorEpiEnd < LookBackDt) then output &OUTFILE._&P;
           %if %upcase(&DEBUG.) eq Y %then %do;
              else output RX_&DrugCategory._epi_excl_&P;
           %end;
       end;

       %if %upcase(&DEBUG.) eq N %then %do;
          drop LookBackDt PriorEpiEnd EStart;
       %end;

     run;

  %END;  /* END PARTITION LOOP */


%put ;
%put ************** END PARTITION LOOP PROCESSING ******************;
%put ;

%mend Incident_Rx_Episodes;

* End of incident_rx_episodes.sas;