/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: prev_summ.sas                                                                       |
|     Macro for prevalent Summary Tables summarization and finalization of tables                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: January 26, 2016                                                                            |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       Utilization (Dx & Px) temporary tables split by years                                         |
|   Program outputs:                                                                                  |
|       Final prevalent table written                                                                 |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

%*** Macro to summarize utilization (Dx and Px) tables;
%macro prev_summ_finalize(
  InFile= ,           /* Source input file name                                 */
  OutFile=,           /* Source output file name                                */
  EventVar=,          /* Name of variable for counting events                   */
  EventVarLength=,    /* Length of output EventVar                              */
  EventVarLabel=,     /* Label for output EventVar                              */
  SummVar=,           /* (OPTIONAL) Name of variable(s) that need to be summed  */
  Setting=,           /* Y=is a stratifier or N=is NOT a stratifier             */
  StratVars=,         /* Name of variable(s) that serve as stratifiers          */
  LookupTbl=,         /* Name of lookup table for descriptions                  */
  DescripPairLength=  /* Four parts separated by "|", e.g. "srt_descrip|DxName|3|Diagnosis Name"
    1) Name of source variable from lookup table
    2) Name of renamed output variable
    3) Length of output variable
    4) Label of output variable                                                  */
  );
  %*** Loop through all years;
  %local Year;
  %do Year=&DP_StartYear. %to &DP_EndYear.;
    %*** Aggregate the table, across the classification variables of
    *** Age_Group, Sex, Period, and Code.  For each combination of these variables, per setting, perform the following:
    *** 1 Count the number of distinct PatIDs and set Members_Setting to this value.
    *** 2 Count the number of rows and set Events_Setting to this value.;
    %** DEV Note: order group-by vars from most to least # of distinct values for best performance;
    proc sql _method;
      create table SettingAggregates&Year. as
      select %InsertChar(&StratVars.), &EventVar. length=&EventVarLength.
        , count(distinct Patkey) as AN_Members, count(*) as AN_Events
        , count(distinct ifn(Setting="IP",PatKey,.)) as IP_Members, sum(Setting="IP") as IP_Events
        , count(distinct ifn(Setting="ED",PatKey,.)) as ED_Members, sum(Setting="ED") as ED_Events
        , count(distinct ifn(Setting="AV",PatKey,.)) as AV_Members, sum(Setting="AV") as AV_Events
      from &Infile.&Year.
      group by %InsertChar(&EventVar. &StratVars.)
      ;
    quit;

    %*** Transform table to create separate rows per setting, creating variables: Setting, Members, Events;
    data Final_Aggregates&Year.(keep=&EventVar. AgeTimePeriodGroupKey Sex Setting Members Events sortedby=&EventVar. &StratVars.);
      set SettingAggregates&Year.;
      array MemberCounts [*] AN_Members IP_Members ED_Members AV_Members;
      array EventCounts  [*] AN_Events IP_Events ED_Events AV_Events;
      array SettingNames [4] $ _temporary_ ("AN", "IP", "ED", "AV");
      do _i=1 to dim(EventCounts);
        if EventCounts[_i] > 0 then do;
          Setting=SettingNames[_i];
          Members=MemberCounts[_i];
          Events=EventCounts[_i];
          output;
        end;
      end;
      informat _all_;
    run;
  %end;

  %** Combine the rows from above into a single table.
  *** Set Age_Group and Age_Group_ID variables.
  *** Link the resulting table with the appropriate description lookup table, on the basis of Lookup.Code = DX_#_Digit.Code
  ***   to get the Lookup.Srt_Descrip variable, renaming this variable to DxName
  *** Finalize table and write to DPLocal;
  proc sql _method;
    create table &OutFile. as
    select put(data.AgeTimePeriodGroupKey,$AgeGroupID_to_Value.) as Age_Group length=5 format=$5. label="Age Group"
      , data.Sex length=1 format=$1. label="Sex"
      , data.Period length=4 format=$4. label="Year"
      , data.&EventVar. length=&EventVarLength. format=$&EventVarLength.. label="&EventVarLabel."
      , lk.%scan(&DescripPairLength.,1,%str(|)) as %scan(&DescripPairLength.,2,%str(|)) length=%scan(&DescripPairLength.,3,%str(|))
      format=$%scan(&DescripPairLength.,3,%str(|)). label="%scan(&DescripPairLength.,4,%str(|))"
      , data.Setting length=2 format=$2. label="Setting"
      , data.Members length=8 format=15. label="# of Members"
      , data.Events length=8 format=15. label="# of Events"
      , input(put(data.AgeTimePeriodGroupKey,$AgeGroupID_to_True_ID.),2.) as Age_Group_ID length=3 format=2. label="ID"
    from (
      %do year=&DP_StartYear. %to &DP_EndYear.;
        select *, "&year." as Period
        from Final_Aggregates&Year.
        %if &Year. NE &DP_EndYear. %then %do;
          union all
        %end;
      %end;) as data
      , &LookupTbl. as lk
    where data.&EventVar. = lk.Code
    having %scan(&DescripPairLength.,2,%str(|)) NE " "
    order by Age_Group_ID, Sex, Period, &EventVar., Setting
    ;
  quit;
%mend;


* End of prev_summ_finalize.sas;