/*------------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: tableextract.sas                                                                       |
|     Macro to extract a table                                                                           |
|--------------------------------------------------------------------------------------------------------|
|                                                                                                        |
|   Version: Revised 1.0                                                                                 |
|--------------------------------------------------------------------------------------------------------|
|                                                                                                        |
|   Contact:                                                                                             |
|       Mini-Sentinel Coordinating Center                                                                |
|       info@mini-sentinel.org                                                                           |
\*------------------------------------------------------------------------------------------------------*/

%** MODIFICATIONS
 MOD: 9/24/15 -MR: 1) Tweak 1st set statement to read just variable attributes and not data rows
                   2) Tweak FilterCode to be IF clause rather than WHERE clause
                   3) Tweak Year and Qtr calculations so that function calls are made only once
                   4) Use global macro variables DP_NumYears, DP_StartQuarter and DP_EndQuarter
*** 9/28/15 - RR  (1) Renamed parm CovKeyVar to CovKeyVars to allow 0, 1, or 2 coverage key variables
***               (2) On reading hash table dataset, added KEEP= dataset option
***               (3) Moved subsetting IF code to immediately below the SET statement for the MSCDM Table being read
***               (4) Enabled optional use of any # of coverage key variables, including none
*** 10/5/15  - RR (1) Added ability to insert SORTEDBY attribute to output datasets
*** 10/7/15  - RR (1) Added ReplaceVarPairs parm to make SORTEDBY attribute more extensible for output datasets
*** 10/13/15  -RR (1) Determine SORTEDBY attribute for input dataset and store in macro variable for output dataset(s);
*** 12/04/15 - RR (1) Added option to create a view as a single (non-split) file
*** 12/08/15 - RR (1) In main data step, set temporary variable lengths explicitly;
***          - MR (1) Ensure that length of CovKeyVars is only set when CovKeyVars are requested;
***          - MR (2) Ensure SortedBy info is kept for all cases of OutFileNames;
*** 1/28/16  - RR (1) Correct management of SortedBy variables;
*** 1/29/16  - MR (1) Set four new macro variables to be local: NumSortedByWords repl ReplacedSortedBy targ;
;
%macro TableExtract(
  MSCDMTable=,        /* (REQUIRED) Name of MSCDM (or other table to be read and modified              */
  TableInVars=,       /* (REQUIRED) Maximum list of variables to be read from MSCDM table              */
  EventDate=,         /* (REQUIRED) Utilization event date variable                                    */
  FilterCode=,        /* (REQUIRED) WHERE expression code for filtering in observations                */
  TransformCode=,     /* (REQUIRED) DATA STEP code for transformations/recoding/cleaning, etc.         */
  ComputeAge=Y,       /* (REQUIRED) whether or not to compute age keys                                 */
  DemHashTable=,      /* (REQUIRED) Name of Demographic derivative table to be read                    */
  HashVarKeys=,       /* (REQUIRED) Key variables for linking in hash join                             */
  HashVarsRead=,      /* (REQUIRED) Variables from Hash table to be placed in hash memory              */
  CovKeyVars=,        /* (OPTIONAL) Numeric value containing bitwise years of enrollment coverage      */
  OutName=,           /* (REQUIRED) Output file name (or name stub for split files)                    */
  OutType=D,          /* (REQUIRED) Output file type (D=dataset, V=view)                               */
  TableOutVars=,      /* (REQUIRED) Maximum list of variables to be written out                        */
  Lengths=,           /* (OPTIONAL) DATA STEP variables and lengths that require adjustment            */
  SplitTime=N,        /* (REQUIRED) Time split intervals (Y=year, Q=quarters, N=no splitting           */
  ReplaceVarPairs=    /* (OPTIONAL) Pairs of replacement variable names, paired with "|", blank delimited */
  );
  ** From left to right, for SORTEDBY variables, variable names can be replaced by the replacement variables
  ** in ReplaceVarPairs but each variable must be in the TableOutVars list;

  %********************************************* BEGIN Parameter Checking ********************************************;
  %* MOD 12/4/15 Views can be created ONLY when no splits are requested;
  %* 1) Validate individual parms that will be cross-checked for their valid values;
  %ValidateParm(parm=OutType, vals=DV, LogMessageStub=);
  %ValidateParm(parm=SplitTime, vals=NQY, LogMessageStub=);
  %* 2) Validate relationship between the parms;

  %*    Force parms to upper case;
  %let OutType=%upcase(&OutType.);
  %let SplitTime=%upcase(&SplitTime.);
  %let TableInVars=%upcase(&TableInVars.);
  %let EventDate=%upcase(&EventDate.);
  %let TableOutVars=%upcase(&TableOutVars.);
  %let ReplaceVarPairs=%upcase(&ReplaceVarPairs.);

  %*   Validate relationship, then if necessary, print error message and abort;
  %if &OutType.=V and &SplitTime. NE N %then %do;
    %put ****************************************************************************************** ;
    %put * You requested output Views (&=OutType.) and Split Files (&=SplitTime.);
    %put * This is not permitted.  Program will abort.;
    %put ****************************************************************************************** ;
    %abort cancel ;
  %end;
  %********************************************** END Parameter Checking *********************************************;

  %*************************************** BEGIN Manage the SORTEDBY attribute ***************************************;
  %local _SortedBy ReplacedSortedBy SortedBy wct wd prct NumSortedByWords targ repl SortedByExpr _OutFileNames year qtr CKV ;
  %let _SortedBy=%retrieve_sortedby(DSName=&MSCDMTable.,FullExpr=N);
  %put Initial &=_SortedBy.; %* QC put length of sortedby macro var = %length(&_SortedBy.) ;
  %*** If SORTEDBY is not blank then continue;
  %*** Initialize final SortedBy parm;
  %let SortedBy=;
  %if %length(&_SortedBy.) > 0 %then %do;
    %*put QC >>> Entered processing;
    %*** Force _SortedBy parm to upper-case;
    %let _SortedBy=%upcase(&_SortedBy.);
    %*** Replace SortedBy variables with any replacement variable;
    %let ReplacedSortedBy=;
    %*** Loop through each found word in the SORTEDBY value;
    %*** Put # of variables from SORTED by into macro var;
    %let NumSortedByWords=%sysfunc(countw(&_SortedBy.,%str( )));
    %do wct=1 %to &NumSortedByWords.;
      %*** Get one word;
      %let wd=%scan(&_SortedBy.,&wct.,%str( ));
      %*put QC >> &=wct. &=wd. ;
      %*** Test if word is in the ReplaceVarPairs values;
      %if %sysfunc(indexw(&ReplaceVarPairs.,&wd.,%str( |))) > 0 %then %do;
        %*put QC >> Found &=wct. &=wd. in &=ReplaceVarPairs. ;
        %*** Loop through ReplaceVarPairs to find the target and replacement;
        %do prct=1 %to %sysfunc(countw(&ReplaceVarPairs.,%str( )));
          %*** Find the target word to replace;
          %let targ=%scan(%scan(&ReplaceVarPairs.,&prct.,%str( )),1,%str(|));
          %if &wd.=&targ. %then %do;
            %*put QC >> &=wct. Target for &=wd. is &=targ.;
            %*** Get the replacement;
            %let repl=%scan(%scan(&ReplaceVarPairs.,&prct.,%str( )),2,%str(|));
            %*put QC >> &=wct. Replacement for &=wd. is &=repl.;
            %*** Make the replacement if it is in the final list of output vars;
            %if %sysfunc(indexw(%upcase(&TableOutVars.),&repl.)) > 0 %then %do;
              %let ReplacedSortedBy=&ReplacedSortedBy. &repl.; %*put QC IF >> &=wct. &=ReplacedSortedBy. ;
            %end;
          %end;
        %end;
      %end;
      %else %do;
        %*** Insert the variable into the list without any replacement;
        %let ReplacedSortedBy=&ReplacedSortedBy. &wd.; %*put QC >> ELSE &=wct. &=ReplacedSortedBy. ;
      %end;
    %end;
    %*** Check if SortedBy variables are within TableOutVars;
    %sortedby_check(SourceWrds=&ReplacedSortedBy.,CheckWrds=&TableOutVars.,ReturnVar=SortedBy);
    %if &debug.=Y %then %do;
      %put After macro call for Sortedby_Check &=SortedBy.;
    %end;
  %end;
  %*** Create final SortedBy expression;
  %if %length(&SortedBy.) > 0 %then %do;
    %let SortedByExpr=%str(SORTEDBY=&SortedBy.);
  %end;
  %else %do;
    %let SortedByExpr=;
  %end;
  %put;
  %if &debug.=Y %then %do;
    %put Replaced &=SORTEDBY. and Final SortedBy Expression = >&SortedByExpr.<;
    %put ;
  %end;
  %*************************************** END Manage the SORTEDBY attribute ***************************************;

  %*************************************** BEGIN Create output file name(s) ****************************************;
  %*** Create only one output file name if NOT using split files;
  %if %upcase(&SplitTime.)= N %then %do;
    %if %upcase(&OutType.)=D %then %do;
      %let _OutFileNames=&OutName.(&SortedByExpr.);
    %end;
    %else %if %upcase(&OutType.)=V %then %do;
      %let _OutFileNames=&OutName.V (&SortedByExpr.)/ view=&OutName.V;
    %end;
  %end;
  %*** Create separate list of output file names if using split files;
  %else %do;
    %* Initialize macro var that will contain all filenames to null;
    %let _OutFileNames=;
    %* Loop through years;
    %do year=&DP_StartYear. %to &DP_EndYear.;
      %** When quarterly splits are requested;
      %if %upcase(&SplitTime.)=Q %then %do;
        %* Loop through quarters, accounting for starting and ending quarters for start year and end year respectively;
        %do qtr=1 %to 4;
          %if ((&Year. > &DP_StartYear.) and (&Year. < &DP_EndYear.))
            or (&Year.=&DP_StartYear. and &qtr. >= &DP_StartQuarter.) or (&Year.=&DP_EndYear. and &qtr. <= &DP_EndQuarter.) %then %do;
            %let _OutFileNames= &_OutFileNames. &OutName.&Year.Q&qtr.(&SortedByExpr.);
            %**put DEBUGGING Within QTR loop: Constructed name=&OutName.&Year.Q&qtr. and _OutFileNames=&_OutFileNames.;
          %end;
        %end;
      %end;
      %else %do;
        %* Loop through years only;
        %let _OutFileNames= &_OutFileNames. &OutName.&Year.(&SortedByExpr.);
        %**put DEBUGGING Within YEAR loop: Constructed name=&OutName.&Year. _OutFileNames=&_OutFileNames.;
      %end;
    %end;
  %end;
  %if &debug.=Y %then %do;
    %put *** DEBUGGING _OutFileNames=&_OutFileNames. (&SortedByExpr.);
  %end;
  %*************************************** END Create output file name(s) ****************************************;

  ****************************** Main data step for extracting ******************************;
  data &_OutFileNames.;
    length &lengths.
    %* MOD 12/8/15: Set length(s) for coverage key variable(s) ;
    %if %length(&CovKeyVars.) > 0 %then %do;
       %do CKV=1 %to %sysfunc(countw(&CovKeyVars.));
         _CovBin&CKV. $&DP_NumYears.
       %end;
    %end;
    ;

    ** Hash join loading ;
    if _n_ eq 0 then do ;
        set &DemHashTable.(keep=&HashVarsRead. &CovKeyVars.) ;
    end ;
    if _n_ eq 1 then do ;
       declare hash PatLink(hashexp:16, dataset:"&DemHashTable.");
       PatLink.definekey(%insertchar(%quotelst(&HashVarKeys.),char=%str(,)));
       PatLink.definedata(%insertchar(%quotelst(&HashVarsRead. &CovKeyVars.),char=%str(,)));
       PatLink.definedone();
   end ;

    ** Read in source dataset ;
    do until(EOF);
      set &MSCDMTable.(keep=&TableInVars.) end=eof;
      *** Filter for valid records;
      if (&FilterCode.) ;
      if PatLink.find()=0 then do;
        %if %upcase(&SplitTime.)=Y or %upcase(&SplitTime.)=Q or %length(&CovKeyVars.) > 0 %then %do ;
           %** Get the year of event ;
            _year=Year(&EventDate.);
        %end ;
        %if %upcase(&SplitTime.)=Q %then %do ;
            %** Get the Quarter of event ;
            _qtr=Qtr(&EventDate.) ;
        %end ;
        %*** Set variable(s) for filtering coverage if required;
        %if %length(&CovKeyVars.) > 0 %then %do;
          *** Loop for number of coverage key variables;
          %do CKV=1 %to %sysfunc(countw(&CovKeyVars.));
            %** Convert to binary coverage values;
            _CovBin&CKV.=put(%scan(&CovKeyVars.,&CKV.,%str( )),binary&DP_NumYears..);
            %** Test binary coverage value per year;
            if substr(_CovBin&CKV.,_year - &DP_StartYear. + 1,1) = "0" then delete;
          %end;
        %end;
        %if %upcase(&ComputeAge.=Y) %then %do;
          ****** Age calculations ****** ;
          ** Age as of event date;
          _AgeEvent=max(0,AgeCalc(Birth_Date, &EventDate.));
          ** Set Age Group key as of event;
          AgeEventGroupKey=put(_AgeEvent,AgeGroup_to_ID.);
          ** Age as of time period beginning;
          _AgeTimePeriod=max(0,AgeCalc(Birth_Date,
            %if %upcase(TimeSplit)=Q %then %do;
              %** Age as of beginning of quarter of event date;
              INTNX("QTR", &EventDate., 0)
            %end;
            %else %do;
              %** Age as of beginning of calendar year;
              INTNX("YEAR", &EventDate., 0)
            %end;
            ));
          * Set Age Group key as of beginning of time period;
          AgeTimePeriodGroupKey=put(_AgeTimePeriod,AgeGroup_to_ID.);
        %end;
        *********** Additional transformation code and creation of new variables;
        &TransformCode.;

        %*********** Selective output statements;
        %if %upcase(&SplitTime.)=N %then %do;
          %** One single output statement: This manages selection of the single output name as follows:
          - If view is selected, the view name is to the left of the / sign
          - If a dataset is selected, the dataset name is to the left of the () dataset options syntax;
          *** No splitting - single dataset or view;
          output %scan(%scan(&_OutFileNames.,1,%str(/)),1,%str(%());
        %end;
        %else %do;
          *** Splitting by period;
          select;
          %* Loop through years;
          %do year=&DP_StartYear. %to &DP_EndYear.;
            %** When quarterly splits are requested;
            %if %upcase(&SplitTime.)=Q %then %do;
              %* Loop through quarters, accounting for starting and ending quarters for start year and end year respectively;
              %do qtr=1 %to 4;
                %if ((&Year. > &DP_StartYear.) and (&Year. < &DP_EndYear.))
                  or (&Year.=&DP_StartYear. and &qtr. >= &DP_StartQuarter.) or (&Year.=&DP_EndYear. and &qtr. <= &DP_EndQuarter.) %then %do;
                  when (&Year.=_year and &qtr.=_qtr) output &OutName.&Year.Q&qtr.;
                %end;
              %end;
            %end;
            %else %do;
              %* Loop through years only;
              when (&Year.=_year) output &OutName.&Year.;
            %end;
          %end;
            otherwise;
          end; %** This ends the SELECT statement;
        %end;
        * Set variables for keeping in output dataset(s);
        keep &TableOutVars.;
      end;
    end;
    stop;
  run;

%mend;

* End of tableextract.sas;